/*===========================================================================

FILE: MIDIPlayer.c

SERVICES: Sample MIDI Player applet showing usage of AEE ISoundPlayer interface

DESCRIPTION
  This file contains the implementation of a MIDI Player applet using the
  AEE ISoundPlayer interface services.

  To use the services of ISoundPlayer interface, perform the following steps:
  --------------------------------------------------------------------------
  1. Create the ISoundPlayer object by calling ISHELL_CreateInstance() with 
     AEECLSID_SOUNDPLAYER as class id. This operation is preferably done in
     the applet event handler
  2. Register the ISoundPlayer callback function. ISoundPlayer interface always
     uses this callback function to notify the applet of any events or status 
     changes. There are primarily four types of callback sent from ISoundPlayer: 
     AEE_SOUNDPLAYER_PLAY_CB, AEE_SOUNDPLAYER_SOUND_CB, AEE_SOUNDPLAYER_STATUS_CB,
     AEE_SOUNDPLAYER_TIME_CB.
     Each callback type, in turn, will
     contain a set of status indications that are only valid to that callback.
     For example, for AEE_SOUNDPLAYER_PLAY_CB, the status type can be 
     AEE_SOUNDPLAYER_SUCCESS, AEE_SOUNDPLAYER_PAUSE, AEE_SOUNDPLAYER_RESUME, etc.
     See  SoundPlayerCBFn() and UpdatePlay() functions in this example.
  3. Set the source of the MIDI media (file/buffer/MIME) using ISOUNDPLAYER_Set() API
  4. In the applet event handler, call
     ISOUNDPLAYER_Play(): To play the MIDI file. Specify full path name of the file.
     ISOUNDPLAYER_Stop(): To stop the playback
     ISOUNDPLAYER_Rewind(): To rewind playback for n milliseconds
     ISOUNDPLAYER_FastForward(): To fast forward playback for n milliseconds
     ISOUNDPLAYER_Pause(): To pause the playback
     ISOUNDPLAYER_Resume: To resume the playback
  5. Always call ISOUNDPLAYER_Stop() before exit. Call it in EVT_APP_STOP.
  6. More on callback:
     1. During playback, every second, applet is notified of AEE_SOUNDPLAYER_TICK_UPDATE status
        via AEE_SOUNDPLAYER_PLAY_CB. This can be used by applet to update the progress 
        of playback.
     2. After rewind, fforward, pause or resume, the dwParam of AEE_SOUNDPLAYER_PLAY_CB
        callback function points to the current position, in milliseconds, of the play.
        Again, applet can use this to update the playback progress
     These mechanisms are demonstrated in this applet for rewind and fforward.

  PUBLIC CLASSES:  
   N/A

INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet

   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR

===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEEAppGen.h"	// AEEApplet structure and Applet services
#include "AEESampleAppIDs.h" // Sample applet id

// Services used by app
#include "AEEMenu.h"    // AEE Menu Services
#include "AEEFile.h"    // AEE File Manager Services
#include "AEESoundPlayer.h" // AEE Sound Player Services

// Applet resource header
#include "MIDIPlayer_res.h"

/*===========================================================================
                      PUBLIC DATA DECLARATIONS
===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
#ifndef DIRECTORY_CHAR
#ifdef AEE_SIMULATOR
#define DIRECTORY_CHAR  '\\'
#define DIRECTORY_STR   "\\"
#else
#define DIRECTORY_CHAR  '/'
#define DIRECTORY_STR   "/"
#endif
#endif 

typedef enum
{
   MIDI_NONE,
   MIDI_PLAY,
   MIDI_PLAY_BUFFER,
   MIDI_PLAY_STREAM,
   MIDI_STOP,
   MIDI_REWIND,
   MIDI_FFORWARD,
   MIDI_PAUSE,
   MIDI_RESUME,
   MIDI_VOLUME_UP,
   MIDI_VOLUME_DOWN,
   MIDI_VOLUME_TICK,
   MIDI_GET_TOTAL_TIME,
   MIDI_SET_TEMPO_SET,
   MIDI_SET_TEMPO_UP,
   MIDI_SET_TEMPO_DOWN,
   MIDI_SET_TUNE_SET,
   MIDI_SET_TUNE_UP,
   MIDI_SET_TUNE_DOWN,
   MIDI_GET_SPEC,
   MIDI_SOUND_DEV
} MIDIOperation;

#define MIDI_MAX_FILES     32
#define MIDI_BUFFER_LEN    35000

//Mandatory structure.
//Structure to contain app data. This is the Applet structure that will be allocated
// for the app. Define this structure if the app needs to store any data that needs
// to be remembered across different events or callbacks. The first element of this
// structure MUST be of type AEEApplet. This is extremely important. Following this
// element, store the app-specific data. Note that AEEApplet structure already exposes
// the most commonly needed interface pointers such as Shell, Display, etc. 
typedef struct _CMIDIApp CMIDIApp;
struct _CMIDIApp
{
   AEEApplet         a;

   IMenuCtl *        m_pIMenu;
   IMenuCtl *        m_pPlayerMenu;

   ISoundPlayer *    m_pISoundPlayer;

   IFileMgr *        m_pIFileMgr;

   uint16            m_wVolume;
   boolean           m_bVolumeTick;
   uint32            m_dwTempo;
   int8              m_u8Tune;

   // Active file info
   char *            m_szFileType;
   MIDIOperation     m_eOperation;
   uint32            m_wTotalTime;
   char *            m_szOperation;

   boolean           m_bPlaying;
   uint32            m_wPlayPos;

   int               m_nActive; // Active file in the player
   char *            m_szFileArray[MIDI_MAX_FILES];

	int               m_cxWidth;
	int               m_cyHeight;
   int               m_nCharSize;
   int               m_NumFiles; // Number of files read into m_szFileArray

   AEESoundPlayerInfo   m_spInfo;
   byte *            m_pBuffer;

   IFile *           m_pIFile;   // File stream
   boolean           m_bStreaming;

   boolean           m_bMainMenu; // Go to main menu

};

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean MIDIApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean MIDIApp_InitAppData(IApplet* po);

// App Helper Functions
// SoundPlayer callback function to be registered with ISoundPlayer interface
static void SoundPlayerCBFn(
   void * pUser, 
   AEESoundPlayerCmd eCBType, 
   AEESoundPlayerStatus eSPStatus,
   uint32         dwParam
);

// Helper functions

static void BuildMusicFilesMenu( CMIDIApp *pMe );
static void BuildPlayerMenu( CMIDIApp *pMe );
static void AddPlayerMenuItem(CMIDIApp * pMe, uint16 wItemID, uint16 wTextID, uint16 wImageID);

static void DrawMainMenu( CMIDIApp * pMe );
static void DrawPlayerMenu( CMIDIApp * pMe );

static boolean HandleMainMenuCmd(CMIDIApp * pMe, uint16 wParam);
static boolean HandlePlayerMenuCmd(CMIDIApp * pMe, MIDIOperation eOp);

static boolean MIDIPlayer_PlayFile(CMIDIApp * pMe);
static boolean MIDIPlayer_PlayBuffer(CMIDIApp * pMe);
static void MIDIPlayer_TimerCB(void * pUser);
static boolean MIDIPlayer_PlayStream(CMIDIApp * pMe);
static void MIDIPlayer_Play(CMIDIApp * pMe);

static void UpdatePlay( CMIDIApp *pMe, AEESoundPlayerStatus eSPStatus, AEESoundPlayerCmdData * pData);
static void DrawStatus( CMIDIApp *pMe, boolean bStatus);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App specific constants
#define MIDI_PLAY_STRING      "PLY"
#define MIDI_STOP_STRING      "STP"
#define MIDI_REWIND_STRING    "REW"
#define MIDI_FFORWARD_STRING  "FF"
#define MIDI_PAUSE_STRING     "PAU"
#define MIDI_RESUME_STRING    "RES"

#define MIDI_TYPE_STRING      "MIDI"
#define MP3_TYPE_STRING       "MP3"
#define UNKNOWN_TYPE_STRING   "UNKN"

#define APP_RES_FILE          "midiplayer.bar"

#define VOLUME_STEP           5
#define MIDI_TEMPO_INC        20
#define MIDI_TUNE_INC         1
#define MIDI_REWIND_MS        5000
#define MIDI_FFORWARD_MS      5000

#define MAX_STRLEN            100

/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/

#if defined(BREW_STATIC_APP)

int         MIDIApp_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj);
static int  MIDIApp_Load(IShell *ps, void * pHelpers, IModule ** pMod);

//
// Constant Data...
//

static const AEEAppInfo    gaiMIDIApp = {AEECLSID_MIDIAPP,APP_RES_FILE,IDS_APP_TITLE,IDB_PLAY,IDB_PLAY,0,0,0};

/*===========================================================================

                      PUBLIC FUNCTION DECLARATIONS

===========================================================================*/

//===========================================================================
//
// 
//
//===========================================================================
PFNMODENTRY MIDIApp_GetModInfo(IShell * ps, AEECLSID ** ppClasses, AEEAppInfo ** pApps, uint16 * pnApps,uint16 * pwMinPriv)
{
   *pApps = (AEEAppInfo *)&gaiMIDIApp;
   *pnApps = 1;
   return((PFNMODENTRY)MIDIApp_Load);
}

//===========================================================================
//
// 
//
//===========================================================================
static int MIDIApp_Load(IShell *ps, void * pHelpers, IModule ** pMod)
{
   return(AEEStaticMod_New((int16)(sizeof(AEEMod)),ps,pHelpers,pMod,MIDIApp_CreateInstance,NULL));
}

#endif  //BREW_STATIC_APP

/*===========================================================================

Function:  MIDIApp_InitAppData()

Description:
		This function initializes the app specific data.

Prototype:
	void MIDIApp_InitAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CMIDIApp*, in this case).

Return Value:
	TRUE: If successful
	FALSE: IF failed

Comments:  None

Side Effects: None

==============================================================================*/
boolean MIDIApp_InitAppData(IApplet* po)
{
	CMIDIApp *pMe = (CMIDIApp*)po;
   int      nAscent, nDescent;

   AEEDeviceInfo  di;

   pMe->m_pISoundPlayer = NULL;
   pMe->m_wVolume = 50;
   pMe->m_bVolumeTick = FALSE;
   pMe->m_dwTempo = 100;
   pMe->m_u8Tune = 0;

   pMe->m_szFileType = UNKNOWN_TYPE_STRING;
   pMe->m_eOperation = MIDI_NONE;
   pMe->m_wPlayPos = pMe->m_wTotalTime = 0;
   pMe->m_bPlaying = FALSE;

   // Get screen pixel count
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);
   pMe->m_cxWidth = di.cxScreen;
	pMe->m_cyHeight = di.cyScreen;

   if ( ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) ||
        ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_ICONVIEWCTL, (void **)&pMe->m_pPlayerMenu) )
   {
	   return FALSE;
   }

   pMe->m_nActive = 0;
   MEMSET(&pMe->m_szFileArray, 0, sizeof(pMe->m_szFileArray));

   IDISPLAY_GetFontMetrics(pMe->a.m_pIDisplay, AEE_FONT_BOLD, &nAscent, &nDescent);
   pMe->m_nCharSize = nAscent + nDescent + 2;

   if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void **)&pMe->m_pIFileMgr))
      return FALSE;

   pMe->m_pBuffer = NULL;

   pMe->m_pIFile = NULL;
   pMe->m_bStreaming = FALSE;

   pMe->m_bPlaying = FALSE;

   pMe->m_bMainMenu = FALSE;

   // Create ISoundPlayer instance
   if ( ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void **)&(pMe->m_pISoundPlayer)) )
      return FALSE;

   // Register SoundPlayer Notification CB Function
   ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, &SoundPlayerCBFn, pMe);

   return TRUE;
}

/*===========================================================================
Function:  MIDIApp_FreeAppData()

Description:
	This function frees the app data. This function is registered with the
	applet framework when the applet is created (inside AEEClsCreateInstance() function).
	This function is called by the app framework when the reference count of the 
	applet reaches zero. This function must free all the app data that has been
	allocated by this app. For ex: if their is data that was remembered when
	the app is suspended and resumed, those data must be freed here.

Prototype:
	void MIDIApp_FreeAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CMIDIApp*, in this case).

Return Value:
	None

Comments:  None

Side Effects: None
==============================================================================*/
void MIDIApp_FreeAppData(IApplet* po)
{
	CMIDIApp * pMe = (CMIDIApp *)po;
   int i = 0;

   for ( i = 0; i < pMe->m_NumFiles; i++ )
   {
      FREEOBJ((void *)pMe->m_szFileArray[i]);
   }
   
   if ( pMe->m_pIMenu )
      IMENUCTL_Release(pMe->m_pIMenu);

   if ( pMe->m_pPlayerMenu )
      IMENUCTL_Release(pMe->m_pPlayerMenu);

   if (pMe->m_pIFile)
      IFILE_Release(pMe->m_pIFile);

   if (pMe->m_pIFileMgr)
      IFILEMGR_Release(pMe->m_pIFileMgr);

   if (pMe->m_pBuffer)
      FREE(pMe->m_pBuffer);

   // Cancel the registered callback and release ISoundPlayer
   ISOUNDPLAYER_RegisterNotify(pMe->m_pISoundPlayer, NULL, NULL);
   if (pMe->m_pISoundPlayer)
      ISOUNDPLAYER_Release(pMe->m_pISoundPlayer);
}

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
#if defined(BREW_STATIC_APP)
int MIDIApp_CreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#else
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
#endif
{
	*ppObj = NULL;

   if(ClsId == AEECLSID_MIDIAPP)
	{
		//Create the applet
      if(AEEApplet_New(sizeof(CMIDIApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)MIDIApp_HandleEvent,(PFNFREEAPPDATA)MIDIApp_FreeAppData))
		{
			//Initialize applet data
         if(MIDIApp_InitAppData((IApplet*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

      }//AEEApplet_New

   }// ClsId == AEECLSID_MIDIAPP

   return(EFAILED);
}

/*===========================================================================

FUNCTION MIDIApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean MIDIApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AppClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
static boolean MIDIApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
    CMIDIApp * pMe = (CMIDIApp *)pi;

 	 AEERect    menuRect;

    SETAEERECT(&menuRect, 0, 0, pMe->m_cxWidth, pMe->m_cyHeight - pMe->m_cyHeight/3);

    switch ( eCode ) 
    {   
         case EVT_APP_START:   // Process Start event
            // Enumerate Files in .\Music directory and constructs a text menu
            BuildMusicFilesMenu(pMe);
            BuildPlayerMenu(pMe);
            DrawMainMenu(pMe);

//            ISOUNDPLAYER_SetSoundDevice(pMe->m_pISoundPlayer, AEE_SOUND_DEVICE_CURRENT, AEE_SOUND_MUTECTL_UNMUTED, AEE_SOUND_MUTECTL_MUTED);
				return TRUE;
            break;

         case EVT_APP_STOP:        // process STOP event
            return (TRUE);
            break;

         case EVT_KEY:	    // process key event
            switch (wParam)
            {
               default:
                  if ( pMe->m_eOperation == MIDI_NONE )
                     return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
                  else
                     return IMENUCTL_HandleEvent(pMe->m_pPlayerMenu, EVT_KEY, wParam, 0);
            } // EVT_KEY

         case EVT_COMMAND:
            if (!pMe->m_pISoundPlayer)
                  return FALSE;

            if ( pMe->m_eOperation == MIDI_NONE )
               return HandleMainMenuCmd(pMe, wParam);
            else
               return HandlePlayerMenuCmd(pMe, (MIDIOperation)wParam);
            break;

    }


    return FALSE;
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION SoundPlayerCBFn

DESCRIPTION
	This is the callback function that is registered with ISoundPlayer interface.
   ISoundPlayer communicates with the applet via this callback only.

PROTOTYPE:
   void SoundPlayerCBFn
   (
      void*                pUser, 
      AEESoundPlayerCmd    eCBType,
      AEESoundPlayerStatus eSPStatus,
      uint32               dwParam
   );

PARAMETERS:
   pUser: A reference to correlate this transaction
   eCBType: Type of ISoundPlayer callback.
   eSPStatus: Status contained in callback. The status depends on the 
            eCBType.
   dwParam: Additional info specific to callback type and status type

   Refer to BREW API Reference for more details.

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void SoundPlayerCBFn
(  void*                pUser, 
   AEESoundPlayerCmd    eCBType,
   AEESoundPlayerStatus eSPStatus,
   uint32               dwParam
)
{
   CMIDIApp* pMe = (CMIDIApp*)pUser;
   AEESoundPlayerCmdData * pData  =  (AEESoundPlayerCmdData *)dwParam;

//   DBGPRINTF("ISoundPlayerCbFn: %d %d %p", eCBType, eSPStatus, pData);

   if ( pMe == 0 )
      return;

   switch(eCBType)
   {
      case AEE_SOUNDPLAYER_PLAY_CB:
           // Received Status Event for Play
           if ( pMe->m_eOperation == MIDI_PLAY     || 
                pMe->m_eOperation == MIDI_REWIND   || 
                pMe->m_eOperation == MIDI_FFORWARD || 
                pMe->m_eOperation == MIDI_PAUSE    || 
                pMe->m_eOperation == MIDI_RESUME   ||
                pMe->m_eOperation == MIDI_STOP  
              )
           {
               UpdatePlay(pMe, eSPStatus, pData);
               if (pMe->m_eOperation == MIDI_NONE)
                  DrawMainMenu(pMe);
               else
                  DrawStatus(pMe, TRUE);
           }
           break;

      case AEE_SOUNDPLAYER_STATUS_CB:
            if (pMe->m_bStreaming)
            {
               pMe->m_bPlaying = pMe->m_bStreaming = FALSE;
               // Streaming happened successfully. Now play the stream.
               if (eSPStatus == AEE_SOUNDPLAYER_SUCCESS)
               {
                  DBGPRINTF("Streaming Successful");
                  MIDIPlayer_Play(pMe);
               }
               else
                  DBGPRINTF("Streaming Failed");

            }
            // Received Status Event for Status
            break;

      case AEE_SOUNDPLAYER_SOUND_CB:
           ISOUNDPLAYER_GetVolume(pMe->m_pISoundPlayer);
           // Received Status Event for Sound
           break;

      case AEE_SOUNDPLAYER_VOLUME_CB:
            if ( eSPStatus == AEE_SOUNDPLAYER_SUCCESS && pData )
            {
               pMe->m_wVolume = pData->wVolume;
               DrawStatus(pMe, TRUE);
            }
            // Received Status Event for Time
            break;

      case AEE_SOUNDPLAYER_TIME_CB:
            if ( eSPStatus == AEE_SOUNDPLAYER_DONE && pData )
            {
               pMe->m_wTotalTime = pData->dwTotalTime / 1000;
               DrawStatus(pMe, TRUE);
               ISOUNDPLAYER_SetVolume(pMe->m_pISoundPlayer, pMe->m_wVolume);
            }

            // Received Status Event for Time
            break;
    }
}

/*===========================================================================

FUNCTION UpdatePlay

DESCRIPTION
	This is called by SoundPlayerCBFn() for AEE_SOUNDPLAYER_PLAY_CB type. It
   handles various playback status types sent by ISoundPlayer.

PROTOTYPE:
   void UpdatePlay
   (
      AEESoundPlayerStatus eSPStatus,
      uint32                    dwParam
   );

PARAMETERS:
   eSPStatus: Status contained in playback callback.
   dwParam: Additional info specific to status type. For example, for status
      type AEE_SOUNDPLAYER_REWIND, this parameter points to the current play
      position in milliseconds.

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void UpdatePlay
(
   CMIDIApp      *pMe,
   AEESoundPlayerStatus eSPStatus,
   AEESoundPlayerCmdData * pData
)
{
   switch (eSPStatus)
   {
        case AEE_SOUNDPLAYER_TICK_UPDATE:
        {
            pMe->m_szOperation = MIDI_PLAY_STRING;
            pMe->m_wPlayPos++;
            if ( pMe->m_bVolumeTick )
            {
               uint16 wVol = pMe->m_wVolume + VOLUME_STEP;
               wVol = wVol > AEE_MAX_VOLUME ? 0 : wVol;
               ISOUNDPLAYER_SetVolume(pMe->m_pISoundPlayer, wVol);
            }
            break;
        }

        case AEE_SOUNDPLAYER_SUCCESS:
            pMe->m_szOperation = MIDI_PLAY_STRING;
            pMe->m_bPlaying = TRUE;
            break;
           
        case AEE_SOUNDPLAYER_RESUME:
        {
            //byte tempMap[8] = { 0x30, 0x38, 0x3c, 0x38, 0x30, 0x00, 0x00, 0x00 };
            pMe->m_wPlayPos = pData ? pData->dwElapsedTime/1000 : pMe->m_wPlayPos;
            pMe->m_szOperation = MIDI_PLAY_STRING;
            break;
        }

        case AEE_SOUNDPLAYER_AUDIO_SPEC:
        {
            if ( pData )
            {
               if ( pData->spSpec.fType == AEE_SOUNDPLAYER_FILE_MP3 )
                  pMe->m_szFileType = MP3_TYPE_STRING;
               else if ( pData->spSpec.fType == AEE_SOUNDPLAYER_FILE_MIDI )
                  pMe->m_szFileType = MIDI_TYPE_STRING;
            }
            break;
        }
      
        case AEE_SOUNDPLAYER_REWIND:
        {
            //byte tempMap[8] = { 0x12, 0x36, 0x7e, 0x36, 0x12, 0x00, 0x00, 0x00 };
            pMe->m_szOperation = MIDI_REWIND_STRING;
            pMe->m_wPlayPos = pData ? pData->dwElapsedTime/1000 : pMe->m_wPlayPos;
            break;
        }
        case AEE_SOUNDPLAYER_FFORWARD:
        {
            //byte tempMap[8] = { 0x48, 0x6c, 0x7e, 0x6c, 0x48, 0x00, 0x00, 0x00 };
            pMe->m_szOperation = MIDI_FFORWARD_STRING;
            pMe->m_wPlayPos = pData ? pData->dwElapsedTime/1000 : pMe->m_wPlayPos;
            break;
        }

        case AEE_SOUNDPLAYER_PAUSE:
        {
            //byte tempMap[8] = { 0x24, 0x24, 0x24, 0x24, 0x24, 0x00, 0x00, 0x00 };
            pMe->m_szOperation = MIDI_PAUSE_STRING;
            pMe->m_wPlayPos = pData ? pData->dwElapsedTime/1000 : pMe->m_wPlayPos;
            break;
        }        

        case AEE_SOUNDPLAYER_FAILURE:
        case AEE_SOUNDPLAYER_ABORTED:
        case AEE_SOUNDPLAYER_DONE:
        {
            if (pMe->m_bPlaying && eSPStatus == AEE_SOUNDPLAYER_FAILURE)
               break;

            //byte tempMap[8] = { 0x3c, 0x3c, 0x3c, 0x3c, 0x3c, 0x00, 0x00, 0x00 };
            pMe->m_bPlaying = FALSE;
            pMe->m_wPlayPos = 0;
            if (pMe->m_bMainMenu)
            {
               pMe->m_bMainMenu = FALSE;
               pMe->m_eOperation =  MIDI_NONE;
            }
            else
               pMe->m_eOperation =  MIDI_STOP;

            pMe->m_szOperation = MIDI_STOP_STRING;
            pMe->m_szFileType = UNKNOWN_TYPE_STRING;
            pMe->m_bVolumeTick = FALSE;

            if (pMe->m_pBuffer)
            {
               FREE(pMe->m_pBuffer);
               pMe->m_pBuffer = NULL;
            }
            
            DBGPRINTF("eSPStatus: %d", eSPStatus);

            ISOUNDPLAYER_SetInfo(pMe->m_pISoundPlayer, NULL);

            break;
        }        
    }
}

static void DrawStatus( CMIDIApp *pMe, boolean bStatus)
{
   AEERect  textRect;
   char     szStr[MAX_STRLEN];
   AECHAR   szBuf[MAX_STRLEN];
   int      yChar = pMe->m_nCharSize;

   IDISPLAY_EraseRgn(pMe->a.m_pIDisplay, 0, 0, pMe->m_cxWidth, pMe->m_cyHeight/2);

   SETAEERECT(&textRect, 0, 0, pMe->m_cxWidth, yChar);

   ISHELL_LoadResString(pMe->a.m_pIShell, APP_RES_FILE, IDS_APP_TITLE, szBuf, MAX_STRLEN);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, textRect.y, &textRect,
                     IDF_ALIGN_CENTER | IDF_ALIGN_TOP | IDF_RECT_FILL | IDF_TEXT_INVERTED );
   
   textRect.y += yChar;

   if ( bStatus )
   {
      SPRINTF( szStr, "%4s: %s", pMe->m_szFileType, STRRCHR(pMe->m_szFileArray[pMe->m_nActive], (int)DIRECTORY_CHAR)+1 );
      STR_TO_WSTR(szStr, szBuf, sizeof(szBuf));
      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 0, textRect.y, &textRect,
                        IDF_ALIGN_LEFT | IDF_ALIGN_TOP );
      textRect.y += yChar;

      SPRINTF(szStr, "%02d:%02d/%02d:%02d/%03d %3s", pMe->m_wPlayPos/60, pMe->m_wPlayPos%60, pMe->m_wTotalTime/60, pMe->m_wTotalTime%60, pMe->m_wVolume, pMe->m_szOperation );
      STR_TO_WSTR(szStr, szBuf, sizeof(szBuf));
      IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, 0, textRect.y, &textRect,
                        IDF_ALIGN_LEFT | IDF_ALIGN_TOP );
      textRect.y += yChar;
   }

   IDISPLAY_Update(pMe->a.m_pIDisplay);
}

static void BuildMusicFilesMenu( CMIDIApp *pMe )
{
   AEERect     menuRect;
   FileInfo    fi;
   uint16      wItemID = 0;
   uint16      yPos = pMe->m_nCharSize;
   AECHAR      *szBuf;

   SETAEERECT(&menuRect, 0, yPos, pMe->m_cxWidth, pMe->m_cyHeight);

   IMENUCTL_Reset(pMe->m_pIMenu); 
   IMENUCTL_SetRect(pMe->m_pIMenu, &menuRect);  
   IMENUCTL_SetTitle(pMe->m_pIMenu, APP_RES_FILE, IDS_MAINMENU_TITLE, NULL);

   // Enumerate all music files and add them to menu
   szBuf = MALLOC(MAX_FILE_NAME);
   if(!szBuf)
      return;

   IFILEMGR_EnumInit(pMe->m_pIFileMgr, "Music", FALSE);
   while ( IFILEMGR_EnumNext(pMe->m_pIFileMgr, &fi) && wItemID < MIDI_MAX_FILES )
   {
      char *szName;

      pMe->m_szFileArray[wItemID] = MALLOC(STRLEN(fi.szName) + 1);
      STRCPY(pMe->m_szFileArray[wItemID], fi.szName);

     // Add only the file name
     szName = STRRCHR(fi.szName, (int)DIRECTORY_CHAR) + 1;
     if ( szName )
      STR_TO_WSTR(szName, szBuf, MAX_FILE_NAME);
     else
      STR_TO_WSTR(fi.szName, szBuf, MAX_FILE_NAME);

     IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, wItemID, szBuf, 0);
     wItemID++;
   }

   FREE(szBuf);

   pMe->m_NumFiles = wItemID;
}

static void AddPlayerMenuItem(CMIDIApp * pMe, uint16 wItemID, uint16 wTextID, uint16 wImageID)
{
   CtlAddItem  mai;

   MEMSET(&mai, 0, sizeof(mai));
	mai.pImage = ISHELL_LoadResImage(pMe->a.m_pIShell, APP_RES_FILE, (int16)wImageID);
   mai.pszResText = APP_RES_FILE;
   mai.wText = wTextID;
   mai.wItemID = wItemID;
   IMENUCTL_AddItemEx(pMe->m_pPlayerMenu, &mai);
   //Release the Image
   if(mai.pImage)
      IIMAGE_Release(mai.pImage);
}

static void BuildPlayerMenu( CMIDIApp *pMe )
{
   AEERect     menuRect;

   int cy = pMe->m_cyHeight/2;
   SETAEERECT(&menuRect, 0, pMe->m_cyHeight - cy, pMe->m_cxWidth, cy);

   IMENUCTL_Reset(pMe->m_pPlayerMenu);
   IMENUCTL_SetRect(pMe->m_pPlayerMenu, &menuRect);  
   IMENUCTL_SetProperties(pMe->m_pPlayerMenu, MP_NO_ARROWS);

   // Play
   AddPlayerMenuItem(pMe, MIDI_PLAY, IDS_PLAY, IDB_PLAY);

   // Play Buffer
   AddPlayerMenuItem(pMe, MIDI_PLAY_BUFFER, IDS_PLAY_BUFFER, IDB_PLAY_BUFFER);

   // Play Stream
   AddPlayerMenuItem(pMe, MIDI_PLAY_STREAM, IDS_PLAY_STREAM, IDB_PLAY_STREAM);

   // Rewind
   AddPlayerMenuItem(pMe, MIDI_REWIND, IDS_REWIND, IDB_REWIND);

   // Stop
   AddPlayerMenuItem(pMe, MIDI_STOP, IDS_STOP, IDB_STOP);

   // FForward
   AddPlayerMenuItem(pMe, MIDI_FFORWARD, IDS_FFORWARD, IDB_FFORWARD);

   // Pause
   AddPlayerMenuItem(pMe, MIDI_PAUSE, IDS_PAUSE, IDB_PAUSE);

   // Volume Up
   AddPlayerMenuItem(pMe, MIDI_VOLUME_UP, IDS_VOLUME_UP, IDB_VOLUME_UP);

   // Volume Up
   AddPlayerMenuItem(pMe, MIDI_VOLUME_DOWN, IDS_VOLUME_DOWN, IDB_VOLUME_DOWN);

   // Volume Tick Toggle
   AddPlayerMenuItem(pMe, MIDI_VOLUME_TICK, IDS_VOLUME_TICK, IDB_VOLUME_TICK);

   // Open Main Menu
   AddPlayerMenuItem(pMe, MIDI_NONE, IDS_MAIN_MENU, IDB_OPEN);
}

static boolean HandleMainMenuCmd(CMIDIApp * pMe, uint16 wParam)
{
   pMe->m_nActive = (int) wParam;

   pMe->m_eOperation = MIDI_STOP;
   pMe->m_szOperation = MIDI_STOP_STRING;
   pMe->m_bPlaying = FALSE;
   pMe->m_wPlayPos = pMe->m_wTotalTime = 0;

   DrawPlayerMenu(pMe);
   IMENUCTL_SetSel(pMe->m_pPlayerMenu, MIDI_PLAY);

   pMe->m_spInfo.eInput = SDT_FILE;
   pMe->m_spInfo.pData = (void *)pMe->m_szFileArray[pMe->m_nActive];
   pMe->m_spInfo.dwSize = 0;
   if ( AEE_SUCCESS != ISOUNDPLAYER_SetInfo(pMe->m_pISoundPlayer, &pMe->m_spInfo) )
      return FALSE;

   ISOUNDPLAYER_GetTotalTime(pMe->m_pISoundPlayer);

   return TRUE;
}

static boolean HandlePlayerMenuCmd(CMIDIApp * pMe, MIDIOperation eOp)
{
   switch( eOp )
   {
	   case MIDI_PLAY:
         if ( pMe->m_bPlaying || EIDLE != ISOUNDPLAYER_GetInfo(pMe->m_pISoundPlayer, NULL) )
            return TRUE;

         MIDIPlayer_PlayFile(pMe);

			return TRUE;
         break;

	   case MIDI_PLAY_BUFFER:
         if ( pMe->m_bPlaying || EIDLE != ISOUNDPLAYER_GetInfo(pMe->m_pISoundPlayer, NULL) )
            return TRUE;

         // Since this operation, usually, requires a large amount of memory,
         // first free ISoundPlayer's audio source and give BREW some time 
         // before you allocate the memory. In the timer callback, call 
         // MIDIPlayer_PlayBuffer()
         ISOUNDPLAYER_SetInfo(pMe->m_pISoundPlayer, NULL);
         ISHELL_SetTimer(pMe->a.m_pIShell, 100, MIDIPlayer_TimerCB, (void *)pMe);

         return TRUE;
         break;

	   case MIDI_PLAY_STREAM:
         if ( pMe->m_bPlaying || EIDLE != ISOUNDPLAYER_GetInfo(pMe->m_pISoundPlayer, NULL) )
            return TRUE;
         
         MIDIPlayer_PlayStream(pMe);

			return TRUE;
         break;

      case MIDI_STOP:
         pMe->m_eOperation = MIDI_STOP;
         ISOUNDPLAYER_Stop(pMe->m_pISoundPlayer);
			return TRUE;
         break;

	   case MIDI_REWIND:
         pMe->m_eOperation = MIDI_REWIND;
         ISOUNDPLAYER_Rewind(pMe->m_pISoundPlayer, MIDI_REWIND_MS);
			return TRUE;
         break;

      case MIDI_FFORWARD:
         pMe->m_eOperation = MIDI_FFORWARD;
         ISOUNDPLAYER_FastForward(pMe->m_pISoundPlayer, MIDI_FFORWARD_MS);
			return TRUE;
         break;

      case MIDI_PAUSE:
         if (!pMe->m_bPlaying)
            return TRUE;
         
         pMe->m_bPlaying = TRUE;

         if ( pMe->m_eOperation != MIDI_PAUSE )
         {
            pMe->m_eOperation = MIDI_PAUSE;
            ISOUNDPLAYER_Pause(pMe->m_pISoundPlayer);
         }
         else
         {
            pMe->m_eOperation = MIDI_PLAY;
            ISOUNDPLAYER_Resume(pMe->m_pISoundPlayer);
         }
			return TRUE;
         break;

      case MIDI_RESUME:
         pMe->m_eOperation = MIDI_RESUME;
         ISOUNDPLAYER_Resume(pMe->m_pISoundPlayer);
			return TRUE;
         break;

      case MIDI_VOLUME_UP:
      {
         uint16 wVol = pMe->m_wVolume + VOLUME_STEP;
         wVol = wVol > AEE_MAX_VOLUME ? 0 : wVol;
         ISOUNDPLAYER_SetVolume(pMe->m_pISoundPlayer, wVol);
			return TRUE;
         break;
      }
      
      case MIDI_VOLUME_DOWN:
      {
         int nVol = pMe->m_wVolume - VOLUME_STEP;
         nVol = nVol < 0 ? AEE_MAX_VOLUME : nVol;
         ISOUNDPLAYER_SetVolume(pMe->m_pISoundPlayer, (uint16)nVol);
			return TRUE;
         break;
      }

      case MIDI_VOLUME_TICK:
         pMe->m_bVolumeTick = !pMe->m_bVolumeTick;
         break;

      case MIDI_NONE:
         if (pMe->m_bPlaying)
         {
            pMe->m_bMainMenu = TRUE;
            ISOUNDPLAYER_Stop(pMe->m_pISoundPlayer);
         }
         else
         {
            pMe->m_szFileType = UNKNOWN_TYPE_STRING;
            pMe->m_eOperation = MIDI_NONE;
            pMe->m_bVolumeTick = FALSE;
            DrawMainMenu(pMe);
         }
			return TRUE;
         break;

      default:
         break;
   } // switch

   return FALSE;
}

static void DrawMainMenu( CMIDIApp * pMe )
{
   IMENUCTL_SetActive(pMe->m_pPlayerMenu, FALSE);
   IDISPLAY_EraseRgn(pMe->a.m_pIDisplay, 0, 0, pMe->m_cxWidth, pMe->m_cyHeight);
   DrawStatus(pMe, FALSE);
   IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
   IMENUCTL_Redraw(pMe->m_pIMenu);
   IDISPLAY_Update(pMe->a.m_pIDisplay);
}

static void DrawPlayerMenu( CMIDIApp * pMe )
{
   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
   IDISPLAY_EraseRgn(pMe->a.m_pIDisplay, 0, 0, pMe->m_cxWidth, pMe->m_cyHeight);
   DrawStatus(pMe, TRUE);
   IMENUCTL_SetActive(pMe->m_pPlayerMenu,TRUE);
   IMENUCTL_Redraw(pMe->m_pPlayerMenu);
   IDISPLAY_Update(pMe->a.m_pIDisplay);
}

static boolean MIDIPlayer_PlayFile(CMIDIApp * pMe)
{
   pMe->m_spInfo.eInput = SDT_FILE;
   pMe->m_spInfo.pData = (void *)pMe->m_szFileArray[pMe->m_nActive];
   pMe->m_spInfo.dwSize = 0;
   if ( AEE_SUCCESS != ISOUNDPLAYER_SetInfo(pMe->m_pISoundPlayer, &pMe->m_spInfo) )
   {
      DBGPRINTF("PlayFile failed");
      return FALSE;
   }

   MIDIPlayer_Play(pMe);

   return TRUE;
}

static boolean MIDIPlayer_PlayBuffer(CMIDIApp * pMe)
{
   IFile *     pIFile;
   uint32      dwRead;

   pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, pMe->m_szFileArray[pMe->m_nActive], _OFM_READ);
   if (!pIFile)
      return FALSE;

   pMe->m_pBuffer = MALLOC(MIDI_BUFFER_LEN);
   if (!pMe->m_pBuffer)
   {
      IFILE_Release(pIFile);
      return FALSE;
   }

   dwRead = IFILE_Read(pIFile, pMe->m_pBuffer, MIDI_BUFFER_LEN);
   if (!dwRead)
   {
      IFILE_Release(pIFile);
      return FALSE;
   }

   IFILE_Release(pIFile);

   pMe->m_spInfo.eInput = SDT_BUFFER;
   pMe->m_spInfo.pData = (void *)pMe->m_pBuffer;
   pMe->m_spInfo.dwSize = dwRead;
   if (ISOUNDPLAYER_SetInfo(pMe->m_pISoundPlayer, &pMe->m_spInfo))
      return FALSE;

   MIDIPlayer_Play(pMe);

   return TRUE;
}

static boolean MIDIPlayer_PlayStream(CMIDIApp * pMe)
{
   if (pMe->m_pIFile)
   {
      IFILE_Release(pMe->m_pIFile);
      pMe->m_pIFile = NULL;
   }

   pMe->m_pIFile = IFILEMGR_OpenFile(pMe->m_pIFileMgr, pMe->m_szFileArray[pMe->m_nActive], _OFM_READ);
   if (!pMe->m_pIFile)
      return FALSE;

   pMe->m_bStreaming = TRUE;
   // Set stream will make a AEE_SOUNDPLAYER_STATUS_CB when streaming is completed.
   ISOUNDPLAYER_SetStream(pMe->m_pISoundPlayer, (IAStream *)pMe->m_pIFile);

   return TRUE;
}

static void MIDIPlayer_TimerCB(void * pUser)
{
   CMIDIApp * pMe = (CMIDIApp *)pUser;

   if (!pUser)
      return;

   MIDIPlayer_PlayBuffer(pMe);
}


static void MIDIPlayer_Play(CMIDIApp * pMe)
{
//   pMe->m_bPlaying = TRUE;
   pMe->m_eOperation = MIDI_PLAY;
   ISOUNDPLAYER_Play(pMe->m_pISoundPlayer);
}

/*========================End of File ================================================*/
